.TH PSGL  GL  "Feb. 3 1991"
.SH NAME
psgl \- GL to EPS converter.
.SH DESCRIPTION
The
.B psgl
package intercepts all GL calls from an application program
and converts these to encapsulated postscript (EPS) commands.
EPS commands are written to a 8x10 postscript page
while preserving aspect ratio.
The EPS %%BoundingBox command is also written for use by
latex, psview, lpr or other programs.
.sp
The
.B psgl
package can be activated in two ways :
.IP 1.
the user can press the
"leftctrl-shift-printscrn" or "rightctrl-shift-printscrn"
sequence.
This prompts the
.B psgl
package to intercept and cache all GL calls until the next
call to "swapbuffers()". 
(see the ASSUMPTIONS section for details).
After "swapbuffers()" is called the EPS file is generated
and the program returns to normal execution mode.
.IP 2.
the application program can initiate the
.B psgl
package directly by calling psgl_open (fname) with string fname 
as the name of a  writeable file.
This directs
.B psgl
to intercept and cache all GL calls until the next
call to "swapbuffers()".
After "swapbuffers()" is called the EPS file is generated
and the application returns to normal execution mode.
.LP
.SH ASSUMPTIONS
The
.B psgl
package makes certain assumptions about the structure of the
application program.
These assumptions can best be explained by describing 
some internals.  When prompted by the user (or application), the 
.B psgl
package queues a REDRAW event.
The application program, upon receiving the REDRAW, 
should handle it appropriately;
i.e. by refreshing the display.
At this point, the
.B psgl
code is enabled and intercepts relevant GL calls up to and
including the next call to
.sp
swapbuffers().
.sp
Each GL primitive is stored in screen coordinates.
At swap time, the primitives are sorted by depth and output 
back to front in encapsulated postscript format.
.sp
The following template code illustrates how the
.B psgl
package can be used:
.nf
    while(TRUE) {
	while(qtest()) {
	    dev=qread(&val);
	    switch(dev) {
		...
		case REDRAW:
	    	    draw_scene ();
		    break;
		...
	    }
	}
	swapbuffers ();
    }
.LP
.fi
.sp
Finally, the single external variable \fBpsgl_enabled\fP
is available to those who wish to have programmatic knowledge
of the package's actions.  This variable is true when and only
when the package is collecting gl output, and should be 
considered \fBread only\fP by applications programmers.
Writing its value will wreak havoc.  An example of its use is
.sp
.nf
#ifdef PSGL
    if (!psgl_enabled)
#endif
        do_something_during_normal_mode_only();
.sp
(The ifdef is required so that the program will compile
when PSGL is not defined.)    
.LP
.SH COMPILATION
To use
.B psgl
add the statement
.sp
#include "psgl.h"
.sp
after the statement that includes "gl/gl.h"
in all files that make GL calls.
Compile these files with -DPSGL to enable the 
.B psgl
routines.
.sp
Compile the file psgl.c with one of -DPI, -DGTX or -DVGX,
depending on your platform.
.sp
Finally, link the resulting .o files with psgl.o and 
any other libraries to make one executable.
.SH FILES
psgl.h
.br
psgl.c
.br
psgl.o
.br
psgl.man
.br
psgl.nroff
.SH LIMITATIONS
This version of
.B psgl
only works with the C interface to GL (i.e., 
libgl_s.a and libgl.a).
.sp
Programs will suffer slight performance degradation when using
.B psgl,
due to the branching that occurs inside every GL routine.
Due to the limited size of the feedback buffer, 
GL routines that generate a potentially large amount 
of output may cause this buffer to overflow.  No easy
workarounds exist at present.
.sp
Due to limitations in postscript, 
gouraud shaded polygons are output as flat shaded,
using a color that is an arithmetic average of the
colors specified to, or computed by, the GL at each
vertex.
.sp
Context-full operations should not be intermixed.
For example, charstr()'s should not occur 
inside bgn*()..end*() pairs, etc.
.sp
.SH BUGS
The package does not yet understand font manager commands,
nor many of the easily added GL routines such as arc*().
.sp
A qenter(REDRAW, winget()) should probably be done rather 
than a qenter(REDRAW, 1).
.sp
Please report any enhancements or bugs to the author.
.SH VERSION
Version 1.0, January 1991.
.SH AUTHOR
Seth Teller (seth@miro.berkeley.edu).
Thanks to robertl@cwi.nl for instigating this man page.

