#ifdef PSGL
/* 
    psgl.h copyright (c) 1991 seth j teller, seth@miro.berkeley.edu.

    permission is granted to modify and redistribute this code in any manner
    as long as this notice is preserved.  all standard disclaimers apply.
*/

/* psgl.h */
/* seth teller, mar 90, seth@miro.berkeley.edu */
/* contains cpp #defines to convert gl code into postscript gl code */

/* in future: should contain prototypes for all emulated gl routines */

extern int psgl_enabled;	/* so user can tell if postscript enabled */
extern int psgl_debug_level;	/* increasing this yields verbosity, ps bounding boxes, etc. */
extern char *psgl_open();	/* prepare for writing postscript objects */

#ifndef PSGLCFILE

#define pnti(x, y, z)	    Pnt((float)x, (float)y, (float)z)
#define pnts(x, y, z)	    Pnt((float)x, (float)y, (float)z)
#define pnt2(x,y)	    Pnt(x, y, 0.0)
#define pnt2i(x, y)	    Pnt((float)x, (float)y, 0.0)
#define pnt2s(x, y)	    Pnt((float)x, (float)y, 0.0)

#define drawi(x, y, z)	    Draw((float)x, (float)y, (float)z)
#define draws(x, y, z)	    Draw((float)x, (float)y, (float)z)
#define draw2(x,y)	    Draw(x, y, 0.0)
#define draw2i(x, y)	    Draw((float)x, (float)y, 0.0)
#define draw2s(x, y)	    Draw((float)x, (float)y, 0.0)

#define rdri(dx, dy, dz)    Rdr((float)dx, (float)dy, (float)dz)
#define rdrs(dx, dy, dz)    Rdr((float)dx, (float)dy, (float)dz)	
#define rdr2(dx, dy)	    Rdr(dx, dy, 0.0)
#define rdr2i(dx, dy)	    Rdr((float)dx, (float)dy, 0.0)
#define rdr2s(dx, dy)	    Rdr((float)dx, (float)dy, 0.0)

#define arci(x, y, r, s, e) Arc((Coord)x, (Coord)y, (Coord)r, (Angle)s, (Angle)e)
#define arcs(x, y, r, s, e) Arc((Coord)x, (Coord)y, (Coord)r, (Angle)s, (Angle)e)

#define arcfi(x, y, r, s, e) Arcf((Coord)x, (Coord)y, (Coord)r, (Angle)s, (Angle)e)
#define arcfs(x, y, r, s, e) Arcf((Coord)x, (Coord)y, (Coord)r, (Angle)s, (Angle)e)

#define circi(x, y, r)	    Circ((Coord)x, (Coord)y, (Coord)r)
#define circs(x, y, r)	    Circ((Coord)x, (Coord)y, (Coord)r)

#define circfi(x, y, r)	    Circf((Coord)x, (Coord)y, (Coord)r)
#define circfs(x, y, r)	    Circf((Coord)x, (Coord)y, (Coord)r)

#define cmovi(x, y, z)	    Cmov((float)x, (float)y, (float)z)
#define cmovs(x, y, z)	    Cmov((float)x, (float)y, (float)z)
#define cmov2(x, y)	    Cmov(x, y, 0.0)
#define cmov2i(x, y)	    Cmov((float)x, (float)y, 0.0)
#define cmov2s(x, y)	    Cmov((float)x, (float)y, 0.0)

#define recti(x1,y1,x2,y2)  Rect((float)x1,(float)y1,(float)x2,(float)y2)
#define rectfi(x1,y1,x2,y2) Rectf((float)x1,(float)y1,(float)x2,(float)y2)
#define rectfs(x1,y1,x2,y2) Rectf((float)x1,(float)y1,(float)x2,(float)y2)

#define feedback Feedback
#define endfeedback Endfeedback
#define pick Pick
#define endpick Endpick
#define perspective Perspective
#define ortho Ortho
#define ortho2 Ortho2
#define viewport Viewport
#define popviewport Popviewport
#define setdepth Setdepth
#define lsetdepth Lsetdepth
#define reshapeviewport Reshapeviewport
#define cpack Cpack
#define RGBcolor rgbcolor
#define cmov Cmov
#define charstr Charstr
#define pnt Pnt
#define draw Draw
#define rdr Rdr
#define rect Rect
#define arc Arc
#define arcf Arcf
#define circ Circ
#define circf Circf
#define polf Polf
#define polfi Polfi
#define polfs Polfs
#define polf2 Polf2
#define polf2i Polf2i
#define polf2s Polf2s
#define crv Crv
#define crvn Crvn
#define rcrv Rcrv
#define rcrvn Rcrvn
#define patch Patch
#define rpatch Rpatch
#define bgnpolygon Bgnpolygon
#define endpolygon Endpolygon
#define bgnline Bgnline
#define endline Endline
#define bgnpoint Bgnpoint
#define endpoint Endpoint
#define bgnclosedline Bgnclosedline
#define endclosedline Endclosedline
#define bgntmesh Bgntmesh
#define endtmesh Endtmesh
#define bgnqstrip Bgnqstrip
#define endqstrip Endqstrip
#define swapbuffers Swapbuffers
#define zbuffer Zbuffer
#define backface Backface
#define winopen Winopen
#define qread Qread
/* #define bgntrim Bgntrim */
/* #define endtrim Endtrim */
#define bgnsurface Bgnsurface 
#define endsurface Endsurface 
#define clear Clear
#define backbuffer Backbuffer
#define frontbuffer Frontbuffer

#endif /* NOT PSGLCFILE */

/* GL REPLACEMENT ROUTINE PROTOTYPES */

#if defined(SGIHW_PI) || defined(SGIHW_VGX)
#define FB_TOKEN float
#elif defined(SGIHW_GTX)
#define FB_TOKEN short
#endif

void 
Feedback(
    FB_TOKEN buffer[],
    long size);

long 
Endfeedback(
    FB_TOKEN buffer[]);

void 
Pick(
    short buffer[], 
    long numnames);

long 
Endpick(
    short buffer[]);

void
Perspective(
    Angle fovy,
    float aspect,
    Coord near, Coord far);

void
Ortho(
    Coord left, Coord right, Coord bottom, Coord top, Coord near, Coord far);

void 
Ortho2(
    Coord left, Coord right, Coord bottom, Coord top);

void
Viewport(
    Screencoord left, Screencoord right, Screencoord bottom, Screencoord top);

void
Popviewport();

void
Setdepth(
    Screencoord near, Screencoord far);

void
Lsetdepth(
    long near, long far);

void
Reshapeviewport();

void Cpack(
    unsigned long pack);

void
rgbcolor(
    short red, short green, short blue);

void
Arc(
    Coord x, Coord y, Coord radius,
    Angle startang, Angle endang);

void
Arcf(
    Coord x, Coord y, Coord radius,
    Angle startang, Angle endang);

void
Circ(
    Coord x, Coord y, Coord radius);

void
Circf(
    Coord x, Coord y, Coord radius);

void
Crv(
    Coord geom[4][3]);

void
Crvn(
    long n,
    Coord geom[][3]);

void
Rcrv(
    Coord geom[4][4]);

void
Rcrvn(
    long n,
    Coord geom[][4]);

void
Patch(
    Matrix geomx, Matrix geomy, Matrix geomz);

void
Rpatch(
    Matrix geomx, Matrix geomy, Matrix geomz, Matrix geomw);

void
Cmov(
    Coord x, Coord y, Coord z);

void
Charstr(
    String str);

/* in postscript, points are just zero-length lines */
void
Pnt(
    Coord x, Coord y, Coord z);

void
Draw(
    Coord x, Coord y, Coord z);

void Rdr(
    Coord x, Coord y, Coord z);

void
Rect(
    Coord x1, Coord y1, Coord x2, Coord y2);

void
Rectf(
    Coord x1, Coord y1, Coord x2, Coord y2);

void
Polf(
    long n,
    Coord parray[][3]);

void
Polfi(
    long n,
    Icoord parray[][3]);

void
Polfs(
    long n,
    Scoord parray[][3]);

void
Polf2(
    long n,
    Coord parray[][2]);

void
Polf2i(
    long n,
    Icoord parray[][2]);

void
Polf2s(
    long n,
    Scoord parray[][2]);

void
Bgnpolygon();

void
Endpolygon();

void
Bgnline();

void
Endline();

void
Bgnpoint();

void
Endpoint();

void
Bgnclosedline();

void
Endclosedline();

void
Bgntmesh();

void
Endtmesh();

void
Bgnqstrip();

void
Endqstrip();

void
Zbuffer(
    Boolean bool);

void Backface(
    Boolean b);

long
Winopen(
    String name);

long
Qread(
    short *data);

void
Bgntrim();

void
Endtrim();

void
Bgnsurface();

void
Endsurface();

void
Clear();

void
Backbuffer(
    Boolean b);

void
Frontbuffer(
    Boolean b);

void
Swapbuffers();

#endif /* PSGL */
